@@ -1,5 +1,5 @@
 /* ThreadGroup -- a group of Threads
-   Copyright (C) 1998, 2000, 2001, 2002 Free Software Foundation, Inc.
+   Copyright (C) 1998, 2000, 2001, 2002, 2005  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -48,7 +48,7 @@ import java.util.Vector;
  * @author John Keiser
  * @author Tom Tromey
  * @author Bryce McKinlay
- * @author Eric Blake <ebb9@email.byu.edu>
+ * @author Eric Blake (ebb9@email.byu.edu)
  * @see Thread
  * @since 1.0
  * @status updated to 1.4
@@ -238,16 +238,16 @@ public class ThreadGroup
    * Check whether this ThreadGroup is an ancestor of the specified
    * ThreadGroup, or if they are the same.
    *
-   * @param g the group to test on
+   * @param group the group to test on
    * @return whether this ThreadGroup is a parent of the specified group
    */
-  public final boolean parentOf(ThreadGroup tg)
+  public final boolean parentOf(ThreadGroup group)
   {
-    while (tg != null)
+    while (group != null)
       {
-        if (tg == this)
+        if (group == this)
           return true;
-        tg = tg.parent;
+        group = group.parent;
       }
     return false;
   }
@@ -535,7 +535,7 @@ public class ThreadGroup
    * manner.
    *
    * @param thread the thread that exited
-   * @param exception the uncaught exception
+   * @param t the uncaught throwable
    * @throws NullPointerException if t is null
    * @see ThreadDeath
    * @see System#err
@@ -684,7 +684,7 @@ public class ThreadGroup
   {
     if (groups == null)
       return;
-    System.out.print(indentation + this);
+    System.out.println(indentation + this);
     indentation += "    ";
     int i = threads.size();
     while (--i >= 0)
@@ -718,6 +718,7 @@ public class ThreadGroup
     if (groups == null)
       return;
     threads.remove(t);
+    t.group = null;
     // Daemon groups are automatically destroyed when all their threads die.
     if (daemon_flag && groups.size() == 0 && threads.size() == 0)
       {
